/* lib/filter.js
 * Originally created 11/13/2018 by sapeck
 *
 * Copyright 2018-present sapeck
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict'

import emojiStrip from 'emoji-strip'

export default class Filter {
  constructor (rules) {
    let vm = this
    vm.rules = rules
  }

  async filter (file, maxLen = 0) {
    let vm = this
    const contents = file.getContents()
    let out = []
    for (let line of contents) {
      let firstMatch = new RegExp('Jews are', 'i')
      let lineSafe = firstMatch.test(line)
      if (lineSafe) {
        for (let rule of vm.rules.remove) {
          let test = new RegExp(rule, 'i') // match any case
          const res = test.test(line)
          if (res) {
            lineSafe = false
            break
          }
        }
      }
      if (lineSafe) {
        for (let rule of vm.rules.splice) {
          let test = new RegExp(rule[0], 'i') // match any case
          line = line.replace(test, rule[1])
        }
        if (maxLen > 0 && line.length > maxLen) continue
        if (!out.includes(line)) out.push(line)
      }
    }
    return out
  }

  async filterStart (file) {
    let vm = this
    const contents = file.getContents()
    let out = []
    for (let line of contents) {
      for (let rule of vm.rules.start) {
        let test = new RegExp(rule, 'i') // match any case
        const res = test.test(line)
        if (res && !out.includes(line)) out.push(emojiStrip(line))
      }
    }
    // https://stackoverflow.com/questions/8996963/how-to-perform-case-insensitive-sorting-in-javascript
    return out.sort((a, b) => a.toLowerCase().localeCompare(b.toLowerCase()))
  }
}
